@@ -49,7 +49,7 @@ _wnck_get_cardinal (Screen *screen,
 
   *val = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -57,7 +57,7 @@ _wnck_get_cardinal (Screen *screen,
 			       0, G_MAXLONG,
 			       False, XA_CARDINAL, &type, &format, &nitems,
 			       &bytes_after, (void*)&num);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -94,7 +94,7 @@ _wnck_get_wm_state (Screen *screen,
   wm_state = _wnck_atom_get ("WM_STATE");
   retval = NormalState;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -102,7 +102,7 @@ _wnck_get_wm_state (Screen *screen,
 			       0, G_MAXLONG,
 			       False, wm_state, &type, &format, &nitems,
 			       &bytes_after, (void*)&num);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return retval;
@@ -138,7 +138,7 @@ _wnck_get_window (Screen *screen,
 
   *val = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -146,7 +146,7 @@ _wnck_get_window (Screen *screen,
 			       0, G_MAXLONG,
 			       False, XA_WINDOW, &type, &format, &nitems,
 			       &bytes_after, (void*)&w);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -182,7 +182,7 @@ _wnck_get_pixmap (Screen *screen,
 
   *val = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -190,7 +190,7 @@ _wnck_get_pixmap (Screen *screen,
 			       0, G_MAXLONG,
 			       False, XA_PIXMAP, &type, &format, &nitems,
 			       &bytes_after, (void*)&w);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -226,7 +226,7 @@ _wnck_get_atom (Screen *screen,
 
   *val = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -234,7 +234,7 @@ _wnck_get_atom (Screen *screen,
 			       0, G_MAXLONG,
 			       False, XA_ATOM, &type, &format, &nitems,
 			       &bytes_after, (void*)&a);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -253,15 +253,21 @@ _wnck_get_atom (Screen *screen,
 }
 
 static char*
-text_property_to_utf8 (const XTextProperty *prop)
+text_property_to_utf8 (Display             *display,
+                       const XTextProperty *prop)
 {
+  GdkDisplay *gdkdisplay;
   char **list;
   int count;
   char *retval;
 
   list = NULL;
 
-  count = gdk_text_property_to_utf8_list_for_display (gdk_display_get_default (),
+  gdkdisplay = _wnck_gdk_display_lookup_from_display (display);
+  if (!gdkdisplay)
+    return NULL;
+
+  count = gdk_text_property_to_utf8_list_for_display (gdkdisplay,
                                           gdk_x11_xatom_to_atom (prop->encoding),
                                           prop->format,
                                           prop->value,
@@ -292,7 +298,7 @@ _wnck_get_text_property (Screen *screen,
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   text.nitems = 0;
   if (XGetTextProperty (display,
@@ -300,7 +306,7 @@ _wnck_get_text_property (Screen *screen,
                         &text,
                         atom))
     {
-      retval = text_property_to_utf8 (&text);
+      retval = text_property_to_utf8 (display, &text);
 
       if (text.value)
         XFree (text.value);
@@ -310,7 +316,7 @@ _wnck_get_text_property (Screen *screen,
       retval = NULL;
     }
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   return retval;
 }
@@ -331,7 +337,7 @@ _wnck_get_string_property_latin1 (Screen *screen,
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   str = NULL;
   result = XGetWindowProperty (display,
 			       xwindow, atom,
@@ -339,7 +345,7 @@ _wnck_get_string_property_latin1 (Screen *screen,
 			       False, XA_STRING, &type, &format, &nitems,
 			       &bytes_after, (guchar **)&str);
 
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return NULL;
@@ -376,7 +382,7 @@ _wnck_get_utf8_property (Screen *screen,
 
   utf8_string = _wnck_atom_get ("UTF8_STRING");
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   val = NULL;
   result = XGetWindowProperty (display,
@@ -386,7 +392,7 @@ _wnck_get_utf8_property (Screen *screen,
 			       False, utf8_string,
 			       &type, &format, &nitems,
 			       &bytes_after, (guchar **)&val);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
 
   if (err != Success ||
       result != Success)
@@ -436,7 +442,7 @@ _wnck_get_window_list (Screen  *screen,
   *windows = NULL;
   *len = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -444,7 +450,7 @@ _wnck_get_window_list (Screen  *screen,
 			       0, G_MAXLONG,
 			       False, XA_WINDOW, &type, &format, &nitems,
 			       &bytes_after, (void*)&data);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -484,7 +490,7 @@ _wnck_get_atom_list (Screen  *screen,
   *atoms = NULL;
   *len = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -492,7 +498,7 @@ _wnck_get_atom_list (Screen  *screen,
 			       0, G_MAXLONG,
 			       False, XA_ATOM, &type, &format, &nitems,
 			       &bytes_after, (void*)&data);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -532,7 +538,7 @@ _wnck_get_cardinal_list (Screen  *screen,
   *cardinals = NULL;
   *len = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   result = XGetWindowProperty (display,
 			       xwindow,
@@ -540,7 +546,7 @@ _wnck_get_cardinal_list (Screen  *screen,
 			       0, G_MAXLONG,
 			       False, XA_CARDINAL, &type, &format, &nitems,
 			       &bytes_after, (void*)&nums);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -582,7 +588,7 @@ _wnck_get_utf8_list (Screen *screen,
 
   utf8_string = _wnck_atom_get ("UTF8_STRING");
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   val = NULL;
   result = XGetWindowProperty (display,
@@ -592,7 +598,7 @@ _wnck_get_utf8_list (Screen *screen,
 			       False, utf8_string,
 			       &type, &format, &nitems,
 			       &bytes_after, (void*)&val);
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
 
   if (err != Success ||
       result != Success)
@@ -677,7 +683,7 @@ _wnck_set_utf8_list (Screen  *screen,
       ++i;
     }
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   XChangeProperty (display,
 		   xwindow,
@@ -685,21 +691,21 @@ _wnck_set_utf8_list (Screen  *screen,
 		   utf8_string, 8, PropModeReplace,
 		   (guchar *) flattened->str, flattened->len);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   g_string_free (flattened, TRUE);
 }
 
 void
-_wnck_error_trap_push (void)
+_wnck_error_trap_push (Display *display)
 {
   gdk_error_trap_push ();
 }
 
 int
-_wnck_error_trap_pop (void)
+_wnck_error_trap_pop (Display *display)
 {
-  XSync (_wnck_get_default_display(), False);
+  XSync (display, False);
   return gdk_error_trap_pop ();
 }
 
@@ -823,9 +829,9 @@ _wnck_iconify (Screen *screen,
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XIconifyWindow (display, xwindow, DefaultScreen (display));
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -836,16 +842,18 @@ _wnck_deiconify (Screen *screen,
    * XMapWindow() called on its windows, need to use the
    * GDK functions
    */
+  Display   *display;
   GdkWindow *gdkwindow;
 
+  display = DisplayOfScreen (screen);
   gdkwindow = _wnck_gdk_window_lookup_from_window (screen, xwindow);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   if (gdkwindow)
     gdk_window_show (gdkwindow);
   else
-    XMapRaised (DisplayOfScreen (screen), xwindow);
-  _wnck_error_trap_pop ();
+    XMapRaised (display, xwindow);
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -873,13 +881,13 @@ _wnck_close (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
               root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 #define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
@@ -918,13 +926,13 @@ _wnck_keyboard_move (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
               root,
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -951,13 +959,13 @@ _wnck_keyboard_size (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
               root,
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -991,13 +999,13 @@ _wnck_change_state (Screen  *screen,
   xev.xclient.data.l[3] = _wnck_get_client_type ();
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -1025,13 +1033,13 @@ _wnck_change_workspace (Screen     *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -1063,13 +1071,13 @@ _wnck_activate (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -1097,13 +1105,13 @@ _wnck_activate_workspace (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -1131,13 +1139,13 @@ _wnck_change_viewport (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -1164,13 +1172,13 @@ _wnck_toggle_showing_desktop (Screen  *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
 	      root,
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 char*
@@ -1282,20 +1290,17 @@ _wnck_get_wmclass (Screen *screen,
 {
   Display *display;
   XClassHint ch;
-  char *retval;
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   ch.res_name = NULL;
   ch.res_class = NULL;
 
   XGetClassHint (display, xwindow, &ch);
 
-  _wnck_error_trap_pop ();
-
-  retval = NULL;
+  _wnck_error_trap_pop (display);
 
   if (res_class)
     *res_class = NULL;
@@ -1368,7 +1373,7 @@ _wnck_select_input (Screen *screen,
 
   gdkwindow = _wnck_gdk_window_lookup_from_window (screen, xwindow);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   if (gdkwindow)
     {
       /* Avoid breaking GDK's setup,
@@ -1382,7 +1387,7 @@ _wnck_select_input (Screen *screen,
     }
 
   XSelectInput (display, xwindow, mask);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 /* The icon-reading code is copied
@@ -1400,9 +1405,6 @@ find_largest_sizes (gulong *data,
   while (nitems > 0)
     {
       int w, h;
-      gboolean replace;
-
-      replace = FALSE;
 
       if (nitems < 3)
         return FALSE; /* no space for w, h */
@@ -1579,7 +1581,7 @@ read_rgb_icon (Screen        *screen,
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   type = None;
   data = NULL;
   result = XGetWindowProperty (display,
@@ -1589,7 +1591,7 @@ read_rgb_icon (Screen        *screen,
 			       False, XA_CARDINAL, &type, &format, &nitems,
 			       &bytes_after, (void*)&data);
 
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
 
   if (err != Success ||
       result != Success)
@@ -1648,11 +1650,14 @@ _wnck_cairo_surface_get_from_pixmap (Screen *screen,
   unsigned int w_ret, h_ret, bw_ret, depth_ret;
   XWindowAttributes attrs;
 
+  surface = NULL;
   display = DisplayOfScreen (screen);
 
+  _wnck_error_trap_push (display);
+
   if (!XGetGeometry (display, xpixmap, &root_return,
                      &x_ret, &y_ret, &w_ret, &h_ret, &bw_ret, &depth_ret))
-    return NULL;
+    goto TRAP_POP;
 
   if (depth_ret == 1)
     {
@@ -1665,7 +1670,7 @@ _wnck_cairo_surface_get_from_pixmap (Screen *screen,
   else
     {
       if (!XGetWindowAttributes (display, root_return, &attrs))
-        return NULL;
+        goto TRAP_POP;
 
       surface = cairo_xlib_surface_create (display,
                                            xpixmap,
@@ -1673,6 +1678,9 @@ _wnck_cairo_surface_get_from_pixmap (Screen *screen,
                                            w_ret, h_ret);
     }
 
+TRAP_POP:
+  _wnck_error_trap_pop (display);
+
   return surface;
 }
 
@@ -1717,14 +1725,12 @@ try_pixmap_and_mask (Screen     *screen,
   if (src_pixmap == None)
     return FALSE;
 
-  _wnck_error_trap_push ();
-
   surface = _wnck_cairo_surface_get_from_pixmap (screen, src_pixmap);
 
   if (surface && src_mask != None)
     mask_surface = _wnck_cairo_surface_get_from_pixmap (screen, src_mask);
-
-  _wnck_error_trap_pop ();
+  else
+    mask_surface = NULL;
 
   if (surface == NULL)
     return FALSE;
@@ -1816,7 +1822,7 @@ get_kwm_win_icon (Screen *screen,
   *pixmap = None;
   *mask = None;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   icons = NULL;
   result = XGetWindowProperty (display, xwindow,
 			       _wnck_atom_get ("KWM_WIN_ICON"),
@@ -1826,7 +1832,7 @@ get_kwm_win_icon (Screen *screen,
 			       &type, &format, &nitems,
 			       &bytes_after, (void*)&icons);
 
-  err = _wnck_error_trap_pop ();
+  err = _wnck_error_trap_pop (display);
   if (err != Success ||
       result != Success)
     return;
@@ -2138,9 +2144,9 @@ _wnck_read_icons (Screen        *screen,
     {
       icon_cache->wm_hints_dirty = FALSE;
 
-      _wnck_error_trap_push ();
+      _wnck_error_trap_push (display);
       hints = XGetWMHints (display, xwindow);
-      _wnck_error_trap_pop ();
+      _wnck_error_trap_pop (display);
       pixmap = None;
       mask = None;
       if (hints)
@@ -2308,14 +2314,14 @@ _wnck_get_window_geometry (Screen *screen,
 
   display = DisplayOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   XGetGeometry (display,
                 xwindow,
                 &root_window,
                 &x, &y, &width, &height, &bw, &depth);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   _wnck_get_window_position (screen, xwindow, xp, yp);
 
@@ -2353,13 +2359,13 @@ void _wnck_set_window_geometry (Screen *screen,
   xev.xclient.data.l[3] = width;
   xev.xclient.data.l[4] = height;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display,
               root,
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 void
@@ -2379,13 +2385,13 @@ _wnck_get_window_position (Screen *screen,
   display = DisplayOfScreen (screen);
   root = RootWindowOfScreen (screen);
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XTranslateCoordinates (display,
                          xwindow,
 			 root,
                          0, 0,
                          &x, &y, &child);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   if (xp)
     *xp = x;
@@ -2411,7 +2417,7 @@ _wnck_set_icon_geometry  (Screen *screen,
   data[2] = width;
   data[3] = height;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   XChangeProperty (display,
 		   xwindow,
@@ -2419,7 +2425,21 @@ _wnck_set_icon_geometry  (Screen *screen,
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (guchar *)&data, 4);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
+}
+
+GdkDisplay*
+_wnck_gdk_display_lookup_from_display (Display *display)
+{
+  GdkDisplay *gdkdisplay = NULL;
+
+  gdkdisplay = gdk_x11_lookup_xdisplay (display);
+
+  if (!gdkdisplay)
+    g_warning ("No GdkDisplay matching Display \"%s\" was found.\n",
+               DisplayString (display));
+
+  return gdkdisplay;
 }
 
 GdkWindow*
@@ -2430,15 +2450,11 @@ _wnck_gdk_window_lookup_from_window (Screen *screen,
   GdkDisplay *gdkdisplay;
 
   display = DisplayOfScreen (screen);
-  gdkdisplay = gdk_x11_lookup_xdisplay (display);
+  gdkdisplay = _wnck_gdk_display_lookup_from_display (display);
+  if (!gdkdisplay)
+    return NULL;
 
-  if (gdkdisplay)
-    return gdk_x11_window_lookup_for_display (gdkdisplay, xwindow);
-
-  g_warning ("No GdkDisplay matching Display \"%s\" was found.\n",
-             DisplayString (display));
-
-  return NULL;
+  return gdk_x11_window_lookup_for_display (gdkdisplay, xwindow);
 }
 
 /* orientation of pager */
@@ -2476,7 +2492,7 @@ _wnck_set_desktop_layout (Screen *xscreen,
   data[2] = rows;
   data[3] = _NET_WM_TOPLEFT;
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   XChangeProperty (display,
                    root,
@@ -2484,7 +2500,7 @@ _wnck_set_desktop_layout (Screen *xscreen,
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (guchar *)&data, 4);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 }
 
 typedef struct
@@ -2556,9 +2572,9 @@ static int next_token = 1;
 static void
 _wnck_free_layout_manager (LayoutManager *lm)
 {
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (lm->display);
   XDestroyWindow (lm->display, lm->window);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (lm->display);
 
   g_slice_free (LayoutManager, lm);
 
@@ -2629,7 +2645,7 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   lm->selection_atom = selection_atom;
   lm->manager_atom = _wnck_atom_get ("MANAGER");
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
 
   lm->window = XCreateSimpleWindow (display,
                                     root,
@@ -2643,7 +2659,7 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   XSetSelectionOwner (display, lm->selection_atom,
 		      lm->window, timestamp);
 
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   /* Check to see if we managed to claim the selection. */
 
@@ -2664,10 +2680,10 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   xev.data.l[3] = 0;	/* manager specific data */
   xev.data.l[4] = 0;	/* manager specific data */
 
-  _wnck_error_trap_push ();
+  _wnck_error_trap_push (display);
   XSendEvent (display, root,
               False, StructureNotifyMask, (XEvent *)&xev);
-  _wnck_error_trap_pop ();
+  _wnck_error_trap_pop (display);
 
   layout_managers = g_slist_prepend (layout_managers,
                                      lm);
@@ -2697,7 +2713,7 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
         {
           if (current_token == lm->token)
             {
-              _wnck_error_trap_push ();
+              _wnck_error_trap_push (display);
 
               /* release selection ownership */
               if (XGetSelectionOwner (display, lm->selection_atom) !=
@@ -2710,7 +2726,7 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
                                       None, timestamp);
                 }
 
-              _wnck_error_trap_pop ();
+              _wnck_error_trap_pop (display);
 
               _wnck_free_layout_manager (lm);
               return;
